@@ -16,7 +16,7 @@
 #' Generate parameters for annotation data simulation.
 #' 
 #' Build date: Feb 24, 2022
-#' Last update: Apr 28, 2022
+#' Last update: Jul 10, 2022
 #'
 #' @author Dong Yin
 #'
@@ -27,8 +27,9 @@
 #' the function returns a list containing
 #' \describe{
 #' \item{$map$pop.map}{the map data with annotation information.}
-#' \item{$map$qtn.num}{integer: the QTN number of single trait; vector: the multiple group QTN number of single trait; matrix: the QTN number of multiple traits.}
 #' \item{$map$qtn.model}{the genetic model of QTN such as 'A + D'.}
+#' \item{$map$qtn.index}{the QTN index for each trait.}
+#' \item{$map$qtn.num}{the QTN number for (each group in) each trait.}
 #' \item{$map$qtn.dist}{the QTN distribution containing 'norm', 'geom', 'gamma' or 'beta'.}
 #' \item{$map$qtn.sd}{the standard deviations for normal distribution.}
 #' \item{$map$qtn.prob}{the probability of success for geometric distribution.}
@@ -48,90 +49,97 @@
 #' @export
 #'
 #' @examples
-#' SP <- param.annot(qtn.num = 10)
+#' SP <- param.annot(qtn.num = list(tr1 = 10))
 #' str(SP)
 param.annot <- function(SP = NULL, ...) {
   
   SP.tmp <- list(...)
   
-  SP.map <- list(
-    pop.map = NULL,
-    qtn.num = 10,
-    qtn.model = "A",
-    qtn.dist = list(tr1 = "norm"),
-    qtn.sd = list(tr1 = NA),
-    qtn.prob = list(tr1 = NA),
-    qtn.shape = list(tr1 = NA),
-    qtn.scale = list(tr1 = NA),
-    qtn.shape1 = list(tr1 = NA),
-    qtn.shape2 = list(tr1 = NA),
-    qtn.ncp = list(tr1 = NA),
-    qtn.spot = FALSE,
-    len.block = 5e7,
-    maf = NULL,
-    recom.spot = FALSE,
-    range.hot = 4:6,
-    range.cold = 1:5
-  )
-  
-  group1 <- c("pop.map", "qtn.num", "qtn.model")
-  group2 <- c("qtn.dist", "qtn.sd", "qtn.prob", "qtn.shape", "qtn.scale", "qtn.shape1", "qtn.shape2", "qtn.ncp")
-  group3 <- c("qtn.spot", "len.block", "maf", "recom.spot", "range.hot", "range.cold")
-  
-  for (x in names(SP.tmp)) {
-    if (x %in% names(SP.map)) {
-      SP.map[[x]] <- SP.tmp[[x]]
+  if (is.null(SP$map)) {
+    SP.map <- list(
+      pop.map = NULL,
+      qtn.model = "A",
+      qtn.index = NULL,
+      qtn.num = list(tr1 = 10),
+      qtn.dist = list(tr1 = "norm"),
+      qtn.sd = list(tr1 = NA),
+      qtn.prob = list(tr1 = NA),
+      qtn.shape = list(tr1 = NA),
+      qtn.scale = list(tr1 = NA),
+      qtn.shape1 = list(tr1 = NA),
+      qtn.shape2 = list(tr1 = NA),
+      qtn.ncp = list(tr1 = NA),
+      qtn.spot = FALSE,
+      len.block = 5e7,
+      maf = NULL,
+      recom.spot = FALSE,
+      range.hot = 4:6,
+      range.cold = 1:5
+    )
+    
+    group1 <- c("pop.map", "qtn.model", "qtn.index")
+    group2 <- c("qtn.num", "qtn.dist", "qtn.sd", "qtn.prob", "qtn.shape", "qtn.scale", "qtn.shape1", "qtn.shape2", "qtn.ncp")
+    group3 <- c("qtn.spot", "len.block", "maf", "recom.spot", "range.hot", "range.cold")
+    
+    for (x in names(SP.tmp)) {
+      if (x %in% names(SP.map)) {
+        SP.map[[x]] <- SP.tmp[[x]]
+      }
     }
-  }
-  
-  nTrait <- 1
-  if (is.matrix(SP.map$qtn.num)) {
-    nTrait <- nrow(SP.map$qtn.num)
-  }
-  
-  if (nTrait > 1) {
-    for (x in group2) {
-      SP.map[[x]] <- rep(SP.map[[x]], nTrait)
-      names(SP.map[[x]]) <- paste0("tr", 1:nTrait)
-    }
-  }
-  
-  for (i in 1:nTrait) {
-    if (SP.map$qtn.dist[[i]] == "norm") {
-      SP.map$qtn.sd[[i]] <- 1
-    } else if (SP.map$qtn.dist[[i]] == "geom") {
-      SP.map$qtn.prob[[i]] <- 0.5
-    } else if (SP.map$qtn.dist[[i]] == "gamma") {
-      SP.map$qtn.shape[[i]] <- 1
-      SP.map$qtn.scale[[i]] <- 1
-    } else if (SP.map$qtn.dist[[i]] == "beta") {
-      SP.map$qtn.shape1[[i]] <- 1
-      SP.map$qtn.shape2[[i]] <- 1
-      SP.map$qtn.ncp[[i]] <- 0
+    
+    if (is.null(SP.map$qtn.index)) {
+      nTrait <- length(SP.map$qtn.num)
     } else {
-      stop("QTN effect distribution should be 'norm', 'geom', 'gamma' or 'beta'!")
+      nTrait <- length(SP.map$qtn.index)
     }
-  }
-  
-  if (nTrait == 1 & length(SP.tmp$qtn.num) > 0) {
-    for (x in group2) {
-      SP.map[[x]][[1]] <- rep(SP.map[[x]][[1]], length(SP.tmp$qtn.num))
+    
+    if (nTrait > 1) {
+      for (x in group2) {
+        SP.map[[x]] <- rep(SP.map[[x]], nTrait)
+        names(SP.map[[x]]) <- paste0("tr", 1:nTrait)
+      }
     }
-  }
-  
-  for (x in names(SP.map)) {
-    if (all(is.na(unlist(SP.map[[x]])))) {
-      SP.map[[x]] <- NULL
+    
+    for (i in 1:nTrait) {
+      nGroup <- length(SP.map$qtn.num[[i]])
+      if (length(SP.map$qtn.dist[[i]]) != nGroup) {
+        SP.map$qtn.dist[[i]] <- rep(SP.map$qtn.dist[[i]], nGroup)
+      }
+      for (j in 1:nGroup) {
+        if (SP.map$qtn.dist[[i]][j] == "norm") {
+          SP.map$qtn.sd[[i]][j] <- 1
+        } else if (SP.map$qtn.dist[[i]][j] == "geom") {
+          SP.map$qtn.prob[[i]][j] <- 0.5
+        } else if (SP.map$qtn.dist[[i]][j] == "gamma") {
+          SP.map$qtn.shape[[i]][j] <- 1
+          SP.map$qtn.scale[[i]][j] <- 1
+        } else if (SP.map$qtn.dist[[i]][j] == "beta") {
+          SP.map$qtn.shape1[[i]][j] <- 1
+          SP.map$qtn.shape2[[i]][j] <- 1
+          SP.map$qtn.ncp[[i]][j] <- 0
+        } else {
+          stop("QTN effect distribution should be 'norm', 'geom', 'gamma' or 'beta'!")
+        }
+      }
     }
-  }
-  
-  if (!SP.map$qtn.spot) {
-    SP.map$maf <- NULL
-  }
-  
-  if (!SP.map$recom.spot) {
-    SP.map$range.hot <- NULL
-    SP.map$range.cold <- NULL
+    
+    for (x in names(SP.map)) {
+      if (all(is.na(unlist(SP.map[[x]])))) {
+        SP.map[[x]] <- NULL
+      }
+    }
+    
+    if (!SP.map$qtn.spot) {
+      SP.map$maf <- NULL
+    }
+    
+    if (!SP.map$recom.spot) {
+      SP.map$range.hot <- NULL
+      SP.map$range.cold <- NULL
+    }
+    
+  } else {
+    SP.map <- SP$map
   }
   
   for (x in names(SP.tmp)) {
@@ -149,7 +157,7 @@ param.annot <- function(SP = NULL, ...) {
 #' Generate parameters for genotype data simulation.
 #' 
 #' Build date: Feb 21, 2022
-#' Last update: Apr 28, 2022
+#' Last update: Jul 4, 2022
 #'
 #' @author Dong Yin
 #'
@@ -176,14 +184,19 @@ param.geno <- function(SP = NULL, ...) {
   
   SP.tmp <- list(...)
   
-  SP.geno <- list(
-    pop.geno = NULL,
-    incols = 1, 
-    pop.marker = 1e4,
-    pop.ind = 1e2,
-    prob = NULL,
-    rate.mut = 1e-8
-  )
+  if (is.null(SP$geno)) {
+    SP.geno <- list(
+      pop.geno = NULL,
+      incols = 1, 
+      pop.marker = 1e4,
+      pop.ind = 1e2,
+      prob = NULL,
+      rate.mut = 1e-8
+    )
+    
+  } else {
+    SP.geno <- SP$geno
+  }
   
   for (x in names(SP.tmp)) {
     if (x %in% names(SP.geno)) {
@@ -200,7 +213,7 @@ param.geno <- function(SP = NULL, ...) {
 #' Generate parameters for phenotype data simulation.
 #' 
 #' Build date: Feb 21, 2022
-#' Last update: Apr 28, 2022
+#' Last update: Jul 4, 2022
 #'
 #' @author Dong Yin
 #'
@@ -238,94 +251,99 @@ param.pheno <- function(SP = NULL, ...) {
   
   SP.tmp <- list(...)
   
-  SP.pheno <- list(
-    pop = NULL,
-    pop.ind = 100,
-    pop.rep = 1,
-    pop.rep.bal = TRUE,
-    pop.env = NULL,
-    phe.model = list(tr1 = "T1 = A + E"),
-    phe.h2A = list(tr1 = NA),
-    phe.h2D = list(tr1 = NA),
-    phe.h2GxG = list(tr1 = NULL),
-    phe.h2GxE = list(tr1 = NULL),
-    phe.h2PE = list(tr1 = NA),
-    phe.var = list(tr1 = NA),
-    phe.corA = NULL,
-    phe.corD = NULL,
-    phe.corGxG = NULL,
-    phe.corPE = NULL,
-    phe.corE = NULL
-  )
-  
-  group1 <- c("pop", "pop.ind", "pop.rep", "pop.rep.bal", "pop.env")
-  group2 <- c("phe.model", "phe.h2A", "phe.h2D", "phe.h2GxG", "phe.h2GxE", "phe.h2PE", "phe.var")
-  group3 <- c("phe.corA", "phe.corA", "phe.corGxG", "phe.corGxE", "phe.corPE", "phe.corE")
-  
-  for (x in names(SP.tmp)) {
-    if (x %in% names(SP.pheno)) {
-      SP.pheno[[x]] <- SP.tmp[[x]]
+  if (is.null(SP$pheno)) {
+    SP.pheno <- list(
+      pop = NULL,
+      pop.ind = 100,
+      pop.rep = 1,
+      pop.rep.bal = TRUE,
+      pop.env = NULL,
+      phe.model = list(tr1 = "T1 = A + E"),
+      phe.h2A = list(tr1 = NA),
+      phe.h2D = list(tr1 = NA),
+      phe.h2GxG = list(tr1 = NULL),
+      phe.h2GxE = list(tr1 = NULL),
+      phe.h2PE = list(tr1 = NA),
+      phe.var = list(tr1 = NA),
+      phe.corA = NULL,
+      phe.corD = NULL,
+      phe.corGxG = NULL,
+      phe.corPE = NULL,
+      phe.corE = NULL
+    )
+    
+    group1 <- c("pop", "pop.ind", "pop.rep", "pop.rep.bal", "pop.env")
+    group2 <- c("phe.model", "phe.h2A", "phe.h2D", "phe.h2GxG", "phe.h2GxE", "phe.h2PE", "phe.var")
+    group3 <- c("phe.corA", "phe.corA", "phe.corGxG", "phe.corGxE", "phe.corPE", "phe.corE")
+    
+    for (x in names(SP.tmp)) {
+      if (x %in% names(SP.pheno)) {
+        SP.pheno[[x]] <- SP.tmp[[x]]
+      }
     }
-  }
-  
-  nTrait <- length(SP.pheno$phe.model)
-  if (nTrait == 0) { nTrait <- 1 }
-  
-  if (nTrait > 1) {
-    for (x in group2) {
-      SP.pheno[[x]] <- rep(SP.pheno[[x]], nTrait)
-      names(SP.pheno[[x]]) <- paste0("tr", 1:nTrait)
+    
+    nTrait <- length(SP.pheno$phe.model)
+    if (nTrait == 0) { nTrait <- 1 }
+    
+    if (nTrait > 1) {
+      for (x in group2) {
+        SP.pheno[[x]] <- rep(SP.pheno[[x]], nTrait)
+        names(SP.pheno[[x]]) <- paste0("tr", 1:nTrait)
+      }
     }
-  }
-  
-  for (i in 1:nTrait) {
-    model.split <- unlist(strsplit(SP.pheno$phe.model[[i]], split = "\\s*\\=\\s*"))
-    eff.name <- unlist(strsplit(model.split[2], split = "\\s*\\+\\s*"))
-    eff.name <- unique(eff.name)
-    if (SP.pheno$pop.rep > 1) {
-      SP.pheno$phe.h2PE[[i]] <- 0.1
+    
+    for (i in 1:nTrait) {
+      model.split <- unlist(strsplit(SP.pheno$phe.model[[i]], split = "\\s*\\=\\s*"))
+      eff.name <- unlist(strsplit(model.split[2], split = "\\s*\\+\\s*"))
+      eff.name <- unique(eff.name)
+      if (SP.pheno$pop.rep > 1) {
+        SP.pheno$phe.h2PE[[i]] <- 0.1
+      }
+      for (j in 1:length(eff.name)) {
+        if (eff.name[j] == "A") {
+          SP.pheno$phe.h2A[[i]] <- 0.3
+        } else if (eff.name[j] == "D") {
+          SP.pheno$phe.h2D[[i]] <- 0.1
+        } else if (grepl(pattern = ":", x = eff.name[j])) {
+          eff.split <- unlist(strsplit(eff.name[j], split = ":"))
+          if (all(eff.split %in% c("A", "D"))) {
+            GxG.tmp <- list(0.1)
+            names(GxG.tmp) <- eff.name[j]
+            SP.pheno$phe.h2GxG[[i]] <- c(SP.pheno$phe.h2GxG[[i]], GxG.tmp)
+          } else {
+            GxE.tmp <- list(0.1)
+            names(GxE.tmp) <- eff.name[j]
+            SP.pheno$phe.h2GxE[[i]] <- c(SP.pheno$phe.h2GxE[[i]], GxE.tmp)
+          } # end if (all(eff.split %in% c("A", "D"))) {
+        } # end if (eff.name[j] == "A") {
+      } # end for (j in 1:length(eff.name)) {
+    } # end for (i in 1:nTrait) {
+    
+    for (x in names(SP.pheno)) {
+      if (all(is.na(unlist(SP.pheno[[x]])))) {
+        SP.pheno[[x]] <- NULL
+      }
     }
-    for (j in 1:length(eff.name)) {
-      if (eff.name[j] == "A") {
-        SP.pheno$phe.h2A[[i]] <- 0.3
-      } else if (eff.name[j] == "D") {
-        SP.pheno$phe.h2D[[i]] <- 0.1
-      } else if (grepl(pattern = ":", x = eff.name[j])) {
-        eff.split <- unlist(strsplit(eff.name[j], split = ":"))
-        if (all(eff.split %in% c("A", "D"))) {
-          GxG.tmp <- list(0.1)
-          names(GxG.tmp) <- eff.name[j]
-          SP.pheno$phe.h2GxG[[i]] <- c(SP.pheno$phe.h2GxG[[i]], GxG.tmp)
-        } else {
-          GxE.tmp <- list(0.1)
-          names(GxE.tmp) <- eff.name[j]
-          SP.pheno$phe.h2GxE[[i]] <- c(SP.pheno$phe.h2GxE[[i]], GxE.tmp)
-        } # end if (all(eff.split %in% c("A", "D"))) {
-      } # end if (eff.name[j] == "A") {
-    } # end for (j in 1:length(eff.name)) {
-  } # end for (i in 1:nTrait) {
-  
-  for (x in names(SP.pheno)) {
-    if (all(is.na(unlist(SP.pheno[[x]])))) {
-      SP.pheno[[x]] <- NULL
+    
+    if (nTrait > 1) {
+      if (!is.null(SP.pheno$phe.h2A)) {
+        SP.pheno$phe.corA <- diag(nTrait)
+      }
+      if (!is.null(SP.pheno$phe.h2D)) {
+        SP.pheno$phe.corD <- diag(nTrait)
+      }
+      if (!is.null(SP.pheno$phe.h2GxG)) {
+        SP.pheno$phe.corGxG <- rep(list(diag(nTrait)), length(SP.pheno$phe.h2GxG[[1]]))
+        names(SP.pheno$phe.corGxG) <- names(SP.pheno$phe.h2GxG[[1]])
+      }
+      if (SP.pheno$pop.rep > 1) {
+        SP.pheno$phe.corPE <- diag(nTrait)
+      }
+      SP.pheno$phe.corE <- diag(nTrait)
     }
-  }
-  
-  if (nTrait > 1) {
-    if (!is.null(SP.pheno$phe.h2A)) {
-      SP.pheno$phe.corA <- diag(nTrait)
-    }
-    if (!is.null(SP.pheno$phe.h2D)) {
-      SP.pheno$phe.corD <- diag(nTrait)
-    }
-    if (!is.null(SP.pheno$phe.h2GxG)) {
-      SP.pheno$phe.corGxG <- rep(list(diag(nTrait)), length(SP.pheno$phe.h2GxG[[1]]))
-      names(SP.pheno$phe.corGxG) <- names(SP.pheno$phe.h2GxG[[1]])
-    }
-    if (SP.pheno$pop.rep > 1) {
-      SP.pheno$phe.corPE <- diag(nTrait)
-    }
-    SP.pheno$phe.corE <- diag(nTrait)
+    
+  } else {
+    SP.pheno <- SP$pheno
   }
   
   for (x in names(SP.tmp)) {
@@ -343,7 +361,7 @@ param.pheno <- function(SP = NULL, ...) {
 #' Generate parameters for selection.
 #' 
 #' Build date: Apr 6, 2022
-#' Last update: Apr 30, 2022
+#' Last update: Jul 4, 2022
 #'
 #' @author Dong Yin
 #'
@@ -374,18 +392,23 @@ param.sel <- function(SP = NULL, ...) {
   
   SP.tmp <- list(...)
   
-  SP.sel <- list(
-    pop.sel = NULL,
-    ps = c(0.8, 0.8),
-    decr = TRUE,
-    sel.crit = "pheno",
-    sel.single = "comb",
-    sel.multi = "index",
-    index.wt = c(0.5, 0.5),
-    index.tdm = 1,
-    goal.perc = 0.1,
-    pass.perc = 0.9
-  )
+  if (is.null(SP$sel)) {
+    SP.sel <- list(
+      pop.sel = NULL,
+      ps = c(0.8, 0.8),
+      decr = TRUE,
+      sel.crit = "pheno",
+      sel.single = "ind",
+      sel.multi = "index",
+      index.wt = c(0.5, 0.5),
+      index.tdm = 1,
+      goal.perc = 0.1,
+      pass.perc = 0.9
+    )
+    
+  } else {
+    SP.sel <- SP$sel
+  }
   
   for (x in names(SP.tmp)) {
     if (x %in% names(SP.sel)) {
@@ -402,7 +425,7 @@ param.sel <- function(SP = NULL, ...) {
 #' Generate parameters for reproduction.
 #' 
 #' Build date: Apr 6, 2022
-#' Last update: Apr 29, 2022
+#' Last update: Jul 4, 2022
 #'
 #' @author Dong Yin
 #'
@@ -427,23 +450,28 @@ param.reprod <- function(SP = NULL, ...) {
   
   SP.tmp <- list(...)
   
-  SP.reprod <- list(
-    pop.gen = 2,
-    reprod.way = "randmate",
-    sex.rate = 0.5,
-    prog = 2
-  )
-  
-  if (!is.null(SP.tmp$reprod.way)) {
-    if (SP.tmp$reprod.way == "userped" & is.null(SP.tmp$userped)) {
-      userped <- data.frame(
-        index = 1:200,
-        sir = c(rep(0, 100), rep(1:50, each = 2)),
-        dam = c(rep(0, 100), rep(51:100, each = 2))
-      )
-      SP.reprod$userped <- userped
-    }
-  } 
+  if (is.null(SP$reprod)) {
+    SP.reprod <- list(
+      pop.gen = 1,
+      reprod.way = "randmate",
+      sex.rate = 0.5,
+      prog = 2
+    )
+    
+    if (!is.null(SP.tmp$reprod.way)) {
+      if (SP.tmp$reprod.way == "userped" & is.null(SP.tmp$userped)) {
+        userped <- data.frame(
+          index = 1:200,
+          sir = c(rep(0, 100), rep(1:50, each = 2)),
+          dam = c(rep(0, 100), rep(51:100, each = 2))
+        )
+        SP.reprod$userped <- userped
+      }
+    } 
+    
+  } else {
+    SP.reprod <- SP$reprod
+  }
   
   for (x in names(SP.tmp)) {
     if (x %in% names(SP.reprod)) {
@@ -460,7 +488,7 @@ param.reprod <- function(SP = NULL, ...) {
 #' Generate parameters for global options.
 #' 
 #' Build date: Apr 16, 2022
-#' Last update: Apr 29, 2022
+#' Last update: Jul 4, 2022
 #'
 #' @author Dong Yin
 #'
@@ -471,12 +499,12 @@ param.reprod <- function(SP = NULL, ...) {
 #' the function returns a list containing
 #' \describe{
 #' \item{$replication}{the replication times of simulation.}
-#' \item{seed.sim}{simulation random seed.}
+#' \item{$seed.sim}{simulation random seed.}
 #' \item{$out}{the prefix of output files.}
 #' \item{$outpath}{the path of output files, Simer writes files only if outpath is not 'NULL'.}
 #' \item{$out.format}{'numeric' or 'plink', the data format of output files.}
 #' \item{$pop.gen}{the generations of simulated population.}
-#' \item{out.geno.gen}{the output generations of genotype data.}
+#' \item{$out.geno.gen}{the output generations of genotype data.}
 #' \item{$out.pheno.gen}{the output generations of phenotype data.}
 #' \item{$useAllGeno}{whether to use all genotype data to simulate phenotype.}
 #' \item{$ncpus}{the number of threads used, if NULL, (logical core number - 1) is automatically used.}
@@ -492,24 +520,29 @@ param.global <- function(SP = NULL, ...) {
   
   SP.tmp <- list(...)
   
-  SP.global <- list(
-    replication = 1,
-    seed.sim = runif(1, 0, 100),
-    out = "simer", 
-    outpath = NULL,
-    out.format = "numeric",
-    pop.gen = 2,
-    out.geno.gen = 1:2,
-    out.pheno.gen = 1:2,
-    useAllGeno = FALSE,
-    ncpus = 0,
-    verbose = TRUE
-  )
-  
-  if (!is.null(SP.tmp$pop.gen)) {
-    SP.global$pop.gen <- SP.tmp$pop.gen
-    SP.global$out.geno.gen <- 1:SP.tmp$pop.gen
-    SP.global$out.pheno.gen <- 1:SP.tmp$pop.gen
+  if (is.null(SP$global)) {
+    SP.global <- list(
+      replication = 1,
+      seed.sim = runif(1, 0, 100),
+      out = "simer", 
+      outpath = NULL,
+      out.format = "numeric",
+      pop.gen = 1,
+      out.geno.gen = 1,
+      out.pheno.gen = 1,
+      useAllGeno = FALSE,
+      ncpus = 0,
+      verbose = TRUE
+    )
+    
+    if (!is.null(SP.tmp$pop.gen)) {
+      SP.global$pop.gen <- SP.tmp$pop.gen
+      SP.global$out.geno.gen <- 1:SP.tmp$pop.gen
+      SP.global$out.pheno.gen <- 1:SP.tmp$pop.gen
+    }
+    
+  } else {
+    SP.global <- SP$global
   }
   
   for (x in names(SP.tmp)) {
@@ -527,7 +560,7 @@ param.global <- function(SP = NULL, ...) {
 #' Generate parameters for Simer.
 #' 
 #' Build date: Apr 17, 2022
-#' Last update: Apr 29, 2022
+#' Last update: Jul 4, 2022
 #'
 #' @author Dong Yin
 #'
